/*
 * SienaGauge.java
 *
 * Created on February 15, 2002, 3:55 PM
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gaugeInfrastructure/Siena/SienaGauge.java,v 1.7 2002/07/11 14:34:15 schmerl Exp $
 * $Log: SienaGauge.java,v $
 * Revision 1.7  2002/07/11 14:34:15  schmerl
 * Removed assumption that gauge runs in separate thread (got rid of System.exit on gauge deletion).
 * Allowed empty setup parameters.
 *
 * Revision 1.6  2002/05/23 23:09:41  schmerl
 * Move GaugeImpl into this class, and also wrote the delegators here. This prevents the need for implementing the delegation in each gauge.
 *
 * Revision 1.5  2002/05/15 16:26:56  schmerl
 * Reconciled Owen's changes, and released new version
 *
 * Revision 1.2  2002/01/31 16:39:06  schmerl
 * Added CVS header information
 **/

package edu.cmu.cs.able.gaugeInfrastructure.Siena;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
/** This is the root class for all Siena gauges.
 * It creates a default Siena gauge bus, and has
 * default implementations for managing deletion and creation of
 * the gauges.
 *
 * @author Bradley Schmerl
 * @version 1.2
 */
public abstract class SienaGauge implements GaugeControl {

    /** The bus to report to.
     */
    protected GaugeReportingBus gaugeBus = new SienaGaugeReportingBus ();
    
    /** The implementation of the gauge **/
    protected GaugeImpl impl = null;

    /** The ID of the gauge manager controlling this gauge.
     */    
    public String myManagerID = null;
    GaugeID gaugeID = null;
    
    /** The parameters used to setup this gauge. 
     */    
    protected StringPairVector setupParams = new StringPairVector ();
    /** The mappings of this gauge.
     */    
    protected StringPairVector mappings = new StringPairVector ();
    
    /** Creates new SienaGauge. Used when the mappings and setup parameters are unknown -- the gauge infrastructure queries the gauge manager to find these values.
     * @param gaugeType The type of the gauge to be created.
     * @param modelName The name of the model that the gauge is attached to.
     * @param modelType The type of the model that the gauge is attached to.
     * @param gaugeName The name of the new gauge.
     * @param creatorID The gauge manager that created the gauge.
     */
    public SienaGauge (String modelName, String modelType, String gaugeName, String gaugeType, String creatorID) {
        gaugeID = new GaugeID ();
        // for debugging possible gauge creation race condition (zensoul-4/30/2002)
        Thread.currentThread().setName(gaugeName + "-Thread");
        edu.cmu.cs.able.gaugeInfrastructure.util.Global.debug ("Siena Gauge: current thread is " + Thread.currentThread());
        // end debugging
        gaugeID.uid = Global.generateUniqueID ();
        gaugeID.modelName = modelName;
        gaugeID.modelType = modelType;
        gaugeID.gaugeName = gaugeName;
        gaugeID.gaugeType = gaugeType;
        myManagerID = creatorID;
              
        ((SienaGaugeReportingBus )gaugeBus).registerGauge (this, setupParams, mappings, true, myManagerID);
        edu.cmu.cs.able.gaugeInfrastructure.util.Global.debug ("Siena Gauge: finished constructor 1");
    }

    /** Creates new SienaGauge. Used when the setup parameters and mappings are known.
     * @param gaugeID The ID of the gauge
     * @param creatorID The gauge manager that created the gauge
     * @param setupParams The setup parameters used to create the gauge
     * @param mappings The mappings for values to high-level properties
     */
    public SienaGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        // for debugging possible gauge creation race condition (zensoul-4/30/2002)
        Thread.currentThread().setName(gaugeID.gaugeName + "-Thread");
        edu.cmu.cs.able.gaugeInfrastructure.util.Global.debug ("Siena Gauge: current thread is " + Thread.currentThread());
        // end debugging
        this.gaugeID = gaugeID;
        gaugeID.uid = Global.generateUniqueID ();
//        gaugeID.gaugeType = gaugeType; 
        myManagerID = creatorID;
        
        this.setupParams = setupParams;
        this.mappings = mappings;
        
        ((SienaGaugeReportingBus )gaugeBus).registerGauge (this, this.setupParams, this.mappings, mappings.isEmpty (), myManagerID);
        edu.cmu.cs.able.gaugeInfrastructure.util.Global.debug ("Siena Gauge: finished constructor 2");
    }
    
    /** Processes a delete message from the gauge manager. This is called by the gauge infrastructure
     * when a delete message is sent to the gauge. This method follows the process for
     * deleting a gauge, i.e.:<br>
     *   report the fact that the gauge was deleted to the gauge bus<br>
     *   stop reporting values<br>
     *   don't listen to any more events<br>
     * It should be assumed by the gauge that it will be destroyed after this method is called
     * (destroyed could mean that the JVM running this gauge is stopped)
     * @param gaugeMgrID The ID of the gauge manager requesting deletion.
     */   
    public void processDelete (String gaugeMgrID) {
        finalizeDeletion (gaugeMgrID);
    }
    
    /** Implements the deletion process for gauges. Can be overridden by implementations.
     * @param gaugeMgrID The ID of the gauge manager requesting deletion.
     */    
    public void finalizeDeletion (String gaugeMgrID) {
        Global.debug ("Gauge is finalizing deletion");
        DeletedEvent event = new DeletedEvent (gaugeID);
        event.gaugeMgrID = gaugeMgrID;
        event.status = true;
        gaugeBus.reportDeleted (event);
//        ((SienaGaugeReportingBus )gaugeBus).sienaBus.siena.shutdown ();
        if (impl != null) {
            impl.stopReportingThread ();
        }
        try {
            Thread.sleep (2000);
        }
        catch (InterruptedException e) {
            
        }
    }
    
    /** Finalizes the creation of the gauge. Sends a created event to the bus, and a creation finished event to the gauge manager.
     * @param gaugeMgrID The ID of the gauge manager that attempted to create the gauge.
     * @param successful Whether the creation was successful.
     */    
    public void finalizeCreation (String gaugeMgrID, boolean successful) {
        Global.debug ("Finalizing creation: " + successful);
        CreationFinishedEvent event = new CreationFinishedEvent (this.getGaugeID ());
        event.gaugeMgrID = gaugeMgrID;
        event.status = successful;
        ((SienaGaugeReportingBus )gaugeBus).gaugeSideBus.publishAnnouncement (event);
        if (successful) {
            CreatedEvent ce = new CreatedEvent (this.getGaugeID ());
            ce.status = successful;
            ce.gaugeMgrID = gaugeMgrID;
            ((SienaGaugeReportingBus )gaugeBus).gaugeSideBus.publishAnnouncement (ce);
        }
        else {
//            ((SienaGaugeReportingBus )gaugeBus).sienaBus.siena.shutdown ();
            try {
                Thread.sleep (2000);
            }
            catch (InterruptedException e) {}
            System.out.println (); System.out.println ("Bye bye");
            System.exit (-1);
        }
    }
    
    /** The method called when configuring a gauge.
     * @param configParams The configuration parameters for the gauge as (name, value) pairs.
     * @return <B>true</B> if the gauge could be configured successfully; <B>false</B> otherwise
     */
    public boolean configure (StringPairVector configParams) {
	if (impl != null) 
	  return impl.configure (configParams);
	else
	  return false;
    }
    
    /** Each gauge implementation must have an ID
     * @return The ID of the gauge
     */
    public GaugeID getGaugeID () {
        return gaugeID;
    }    
    
    /** Sets the gauge ID for this gauge.
     * @param gid The ID of the gauge.
     */    
    public void setGaugeID (GaugeID gid) {
        gaugeID = gid;
    }
    
    /** Returns all the gauge values that the gauge reports.
     * @param values This is filled with all the values as a (valueName, propertyName, value) tuple
     * @return <B>true</B> if the values were successfully determined; <B>false</B> otherwise.
     */
    public boolean queryAllValues (GaugeValueVector values) {
	if (impl != null) 
	  return impl.queryAllValues (values);
	else
	  return false;
    }
    
    /** The method called to query the state of the gauge
     * @param setupParams Will be filled with the parameters with which the gauge was setup, as (name, value) pairs.
     * @param configParams Will be filled with the current configuration of the gauge, as (name, value) pairs
     * @param mappings Will be filled with the mappings for the gauge, as (name, propertyName) pairs.
     * @return <B>true</B> if the gauge successfully returns its state; <B>false</B> otherwise
     */
     public boolean queryState (StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
	if (impl != null) 
	  return impl.queryState (setupParams, configParams, mappings);
        else
	  return false;
     }
    
    /** Called when a gauge is queried for a value. This provides an
     * alternative to the gauge reporting events.
     * @param valueName The name of the value to be queried.
     * @return The value corresponding to the name passed in; <B>null</B> indicates that the gauge value could not be determined.
     */
    public String queryValue (String valueName) {
	if (impl != null) 
	  return impl.queryValue (valueName);
	else 
	  return "";
    }
    
}
